//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MAPGENERATOR_H
#define MAPGENERATOR_H

#include <string>
#include <vector>

/** This class creates a map including the buildings (cities, ruins, temples).
  * It does NOT care about player setup and such. This is done in a second step
  * by the CreateScenario class. The parameters for map creation are set by
  * GamePreferencesDialog::fillData().
  *
  * The data is stored in two variables:
  *   - the terrain map stores the terrain information
  *   - the building map stores the information about building placement
  *
  * For historical reasons *cough*, the maps are long character arrays. The
  * definition of the single characters can be found in defs.h; missing
  * buildings are denoted by a space. As a normal user, you will propably never
  * need to worry about the format anyway. It is read and "translated" by the
  * CreateScenario class which kind of supervises the scenario creation.
  *
  * It is possible and will hopefully happen in the future that map generatos
  * with different styles (e.g. islands) are derived from this class.
  */

class MapGenerator
{
    public :
        MapGenerator();
        ~MapGenerator();

        //! Set the number of cities
        int setNoCities(int nocities);

        //! Set the number of ruins
        int setNoRuins(int noruins);

        //! Set the number of temples
        int setNoTemples(int notemples);

        /** Set terrain distribution. If the sum of the percentages is less than
          * 100, the rest is filled with grass.
          */
        void setPercentages(int pwater, int pforest, int pswamp,
                            int phills, int pmountains);

        //! Get number of cities
        int getNoCities() {return d_nocities;}

        //! Get number of ruins
        int getNoRuins() {return d_noruins;}

        //! Get number of temples
        int getNoTemples() {return d_notemples;}
        
        /** Creates a map
          * 
          * Use this function to start off the map generation. This
          * implementation will first distribute the terrain in random patches,
          * normalize it and then distribute the buildings.
          *
          * @param size         the size of the map to be generated
          */
        void MakeMap(int size);

        /** Get the array for the terrain map (shallow copy)
          * 
          * @param size         is set to the size of the generated map
          * @return char array which represents the terrain map
          */
        char* getMap(int& size);
        
        /** Get the buildings map (shallow copy)
          * 
          * @param size         set to the size of the generated map
          * @return char array which represents the building map
          */
        char* getBuildings(int& size);

    private:

        //! Fills the terrain map with grass
        void makePlains();

        //! Spreads the terrain c with the given percentage over the terrain map
        void makeTerrain(char c,int percent);

        /** Distributes cities on the building map. Creates an island if it
          * finds only water or occupied places.
          *
          * @param cities       the number of cities to distribute
          */
        void makeCities(int cities);

        /** Distributes buildings over the map. Creates an island if there are
          * only water or occupied places.
          *
          * @param c            the character for the building
          * @param building     the number of buildings to distribute
          */
        void makeBuildings(char c, int building);

        //! Returns true if position (x,y) is free for buildings
        bool canPutBuilding(int x,int y);

        //! Normalizes the terrain map by removing isolated tiles by chance
        void normalize();

        //Data
        char* d_terrain;
        char* d_building;
        int d_size;
        int d_pswamp, d_pwater, d_pforest, d_phills, d_pmountains;
        unsigned int d_nocities, d_notemples, d_noruins;
};

#endif

// End of file
