//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ACTION_H
#define ACTION_H

#include <string>
#include "defs.h"
#include "stack.h"
#include "fight.h"
#include "city.h"
#include "ruin.h"
#include "temple.h"
#include "xmlhelper.h"

/** The purpose of the action classes is to keep track about what a player has
  * done. This information can e.g. then be sent over the network, so that a
  * networked player then just has to decode and repeat the actions which the
  * remote player has done.
  * 
  * Each action item is derived from the abstract Action class. It has to
  * contain three functions:
  *
  * - A loading constructor (which takes an XML_Helper parameter)
  * - a save function which saves the data
  * - a fill_data function which takes some parameters and with these stores the
  *   data about what happened.
  */


class Action
{
    public:
        Action();
        virtual ~Action();

        //! Returns debug information. Needs to be overwritten by derivatives
        virtual std::string dump()=0;

        //! Save function. See XML_Helper for information about saving.
        virtual bool save(XML_Helper* helper)=0;
        
        /** static load function (see XML_Helper)
          * 
          * Whenever an action item is loaded, this function is called. It
          * examines the stored id and calls the constructor of the appropriate
          * action class.
          *
          * @param helper       the XML_Helper instance for the savegame
          */
        static Action* handle_load(XML_Helper* helper);

        //! Returns the id which identifies the type of the action
        unsigned int get_id() {return d_id;}
        
    protected:
        unsigned int d_id;
};

//-----------------------------------------------------------------------------

class Action_Fight : public Action
{
    public:
        Action_Fight();
        Action_Fight(XML_Helper* helper);
        ~Action_Fight();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(Stack* attacker, Stack* defender,
                Stack* orig_attacker, Stack* orig_defender);

    private:
        unsigned int d_attacker_id, d_defender_id;
        int d_attacker_damages[8], d_defender_damages[8];
};

//-----------------------------------------------------------------------------

class Action_Split : public Action
{
    public:
        Action_Split();
        Action_Split(XML_Helper* helper);
        ~Action_Split();

        std::string dump();
        bool save (XML_Helper* helper);

        bool fill_data(Stack* orig_stack, Stack* added_stack);
            //both stacks are already split!!!
    
    private:
        unsigned int d_orig_stack_id, d_added_stack_id;
        unsigned int d_armies_moved[8];
};

//-----------------------------------------------------------------------------

class Action_Join : public Action
{
    public:
        Action_Join();
        Action_Join(XML_Helper* helper);
        ~Action_Join();

        std::string dump();
        bool save(XML_Helper* helper);
        bool fill_data(Stack* orig, Stack* joining);
    
    private:
        unsigned int d_orig_id, d_joining_id;
};

//-----------------------------------------------------------------------------

class Action_Fight_City : public Action
{
    public:
        Action_Fight_City();
        Action_Fight_City(XML_Helper* helper);
        ~Action_Fight_City();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(std::vector<Stack*> orig_attackers,
                std::vector<Stack*> orig_defenders,
                std::vector<Stack*> attackers,
                std::vector<Stack*> defenders);
        
    private:
        unsigned int d_attacker_ids [Fight::ATTACKER_MAX];
        unsigned int d_defender_ids [Fight::DEFENDER_MAX];
        int d_attacker_damages [Fight::ATTACKER_MAX][8];
        int d_defender_damages [Fight::DEFENDER_MAX][8];
};

//-----------------------------------------------------------------------------

class Action_Move_Step : public Action
{
    public:
        Action_Move_Step();
        Action_Move_Step(XML_Helper* helper);
        ~Action_Move_Step();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(Stack* s, PG_Point dest);
    
    private:
        unsigned int d_stack_id;
        PG_Point d_dest;
};

//-----------------------------------------------------------------------------

class Action_Occupy : public Action
{
    public:
        Action_Occupy();
        Action_Occupy(XML_Helper* helper);
        ~Action_Occupy();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data (City* c);
    
    private:
        unsigned int d_city_id;
};

//-----------------------------------------------------------------------------

class Action_Raze : public Action
{
    public:
        Action_Raze();
        Action_Raze(XML_Helper* helper);
        ~Action_Raze();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data (City* c);
    
    private:
        unsigned int d_city_id;
};

//-----------------------------------------------------------------------------

class Action_Pillage : public Action
{
    public:
        Action_Pillage();
        Action_Pillage(XML_Helper* helper);
        ~Action_Pillage();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(City* c);

    private:
        unsigned int d_city_id;
};
//-----------------------------------------------------------------------------

class Action_Search : public Action
{
    public:
        Action_Search();
        Action_Search(XML_Helper* helper);
        ~Action_Search();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(Ruin* r, Stack* orig_keepers,
                Stack* orig_explorers, Stack* explorers,
                bool searched);
    
    private:
        unsigned int d_ruin_id;
        unsigned int d_stack_id;
        bool d_searched;
        int d_keeper_damages[8];
        int d_seeker_damages[8];
};

//-----------------------------------------------------------------------------

class Action_Temple : public Action
{
    public:
        Action_Temple();
        Action_Temple(XML_Helper* helper);
        ~Action_Temple();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(Temple* t, Stack* s);
    
    private:
        unsigned int d_temple_id;
        unsigned int d_stack_id;
};

//-----------------------------------------------------------------------------

class Action_Upgrade_Defense : public Action
{
    public:
        Action_Upgrade_Defense();
        Action_Upgrade_Defense(XML_Helper* helper);
        ~Action_Upgrade_Defense();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(City* c);

    private:
        unsigned int d_city_id;
};

//-----------------------------------------------------------------------------

class Action_Buy_Production : public Action
{
    public:
        Action_Buy_Production();
        Action_Buy_Production(XML_Helper* helper);
        ~Action_Buy_Production();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(City* c, int prod);

    private:
        unsigned int d_city_id;
        int d_prod;
};

//-----------------------------------------------------------------------------

class Action_Change_Production : public Action
{
    public:
        Action_Change_Production();
        Action_Change_Production(XML_Helper* helper);
        ~Action_Change_Production();

        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data(City* c, int prod);

    private:
        unsigned int d_city_id;
        int d_prod;
};

//-----------------------------------------------------------------------------

//this class will certainly have to be changed a lot, it doesn't handle anything
//else but gold, but I want at least a basic

class Action_Reward : public Action
{
    public:
        Action_Reward();
        Action_Reward(XML_Helper* helper);
        ~Action_Reward();
        
        std::string dump();
        bool save(XML_Helper* helper);

        bool fill_data (Stack* s, fl_Treasure reward);
    
    private:
        unsigned int d_stack_id;
        fl_Treasure d_reward;
};
        
#endif //ACTION_H
