//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMY_H
#define ARMY_H

#include <string>
#include <sigc++/sigc++.h>
#include "armyset_army.h"
#include "xmlhelper.h"

class Armyset;
class Player;

/** Class which describes an army.
  * 
  * This class is used to hold the information about a single unit in the actual
  * game.
  */

class Army : public Armyset_army, public SigC::Object
{
    public:
        /** Default constructor
          * 
          * @param armyset      the armyset which contains the type of the army
          * @param type         the type of the unit (== index in the armyset)
          * @param owner        the player owning this army
          */
        Army(Armyset* armyset, int type, Player* owner);

        //! Copy constructor
        Army(Army& a);

        /** Loading constructor. For internal reasons, the type has to be
          * supplied on creation of this class. That is why this constructor
          * takes more parameters than usual.
          *
          * @param armyset      the armyset which contains the type of this army
          * @param type         the type of this unit (== index in the armyset)
          * @param helper       the XML_Helper instance of the savegame
          */
        Army(Armyset* armyset, int type, XML_Helper* helper);
        ~Army();

        
        //! Set the player owning this army
        void setOwner(Player* owner) {d_owner = owner;}

        //! Set the number of hitpoints of this army
        void setHP(int hp) {d_hp = hp;}

        //! Set the current strength of this army
        void setStrength(int strength) {d_strength = strength;}

        /** Set the grouping information of this army.
          * 
          * Background: When you select a stack on the map, you can toggle the
          * armies which will be moved when you click somewhere else on the map
          * then. Grouped==true means that this army will be among the selected
          * armies.
          */
        void setGrouped(bool grouped){d_grouped = grouped;}

        
        //! Get the unique id of this army
        unsigned int getId() const {return d_id;}

        //! Get the type of this army
        int getType() const {return d_type;}

        //! Get the player owning this army
        Player* getOwner() const {return d_owner;}

        //! Get the image of the army. Internally, this refers to GraphicsCache.
        virtual SDL_Surface* getPixmap();

        //! Get the current number of hitpoints (in opposite to getMaxHp)
        int getHP() const {return d_hp;}

        //! Get the current number of moves (in opposite to getMaxMoves)
        int getMoves() const {return d_moves;}

        //! Get the actual strength of the unit
        int getStrength() const {return d_strength;}

        //! Get the number of experience points of the unit
        double getXP() const {return d_xp;}

        //! Get the level of the army
        int getLevel() const {return d_level;}

        //! Returns whether army is blessed (may only happen once :-))
        bool isBlessed() const {return d_blessed;}

        //! Returns grouping information
        bool isGrouped() const {return d_grouped;}

        //! Returns whether the army is a hero
        virtual bool isHero() {return false;}

        
        /** Heal the unit
          * 
          * @param hp       hitpoints to hal. Set to zero for "natural" healing.
          */
        void heal(int hp = 0);

        /** Damage this unit
          * 
          * @param damageDone       how much damage the unit suffers
          * @return true if the unit has died, else otherwise
          */
        bool damage(int damageDone);

        /** Reduce the number of moves
          * 
          * @param moves            how many movement points have been spent
          */
        void decrementMoves(int moves);

        //! Resets the number of movement points to maximum
        void resetMoves();

        //! Blesses the unit (currently: strength+1)
        void bless();

        //! Increases the XP of the unit
        void gainXp(double n);

        //! Checks whether unit can advance a level
        bool canGainLevel();

        //! Has the unit advance a level
        void gainLevel();

        //! Saves the unit information (see XML_Helper for further info)
        virtual bool save(XML_Helper* helper);
       
        
        //! This signal is raised when the army dies.
        SigC::Signal1<void, Army*> sdying;
        
    protected:
        //! Generic function for saving the army data. Useful for the hero class,
        //  which doesn't need to repeat the save code.
        bool saveData(XML_Helper* helper);
        
        Armyset* d_armyset;
        Player* d_owner;
        unsigned int d_id;
        int d_type;
        int d_hp;
        int d_moves;
        int d_strength;
        double d_xp;            // experience points
        int d_level;
        bool d_blessed;            // if army is blessed then strength is +1
        bool d_grouped;
};

#endif // ARMY_H
