//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMYSETLIST_H
#define ARMYSETLIST_H

#include <string>
#include <map>
#include <sigc++/object_slot.h>
#include "xmlhelper.h"

class Armyset;

/** List of all available armysets
  * 
  * This class contains all needed armysets. You can request an armyset with
  * getArmyset and load one with loadArmyset.
  *
  * Since several classes access this class, it is implemented as a singleton.
  */

class Armysetlist : public std::map<std::string, Armyset*>, public SigC::Object
{
    public:
        //! return the singleton instance of this class
        static Armysetlist* getInstance();

        //! Explicitely delete the singleton instance of this class
        static void deleteInstance();

        /** Request a specific armyset
          * 
          * @param name         the name of the requested armyset
          * @return the requested armyset or 0 if the armyset hasn't been
          * loaded yet.
          */
        Armyset* getArmyset(std::string name);

        /** Load an armyset
          * 
          * @param name         the name of the armyset to be loaded
          * @return true if all went well, false on error.
          */
        bool loadArmyset(std::string name);

        /** Erase an armyset by its name
          *
          * @param name         the name of the armyset
          */
        bool eraseByName(std::string name);

    protected:
        Armysetlist();
        ~Armysetlist();
        
    private:
        //! Callback function for the armyset tag (see XML_Helper)
        bool loadGlobalStuff(std::string tag, XML_Helper* helper);

        //! Callback function for the army tag (see XML_Helper)
        bool loadArmy(std::string tag, XML_Helper* helper);

        
        static Armysetlist* s_instance;
        std::string d_loadName;   // name of the armyset to load, internal use
        int d_loadCounter;
};

#endif // ARMYSETLIST_H

