//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PLAYER_H
#define PLAYER_H

#include <string>
#include <list>
#include <pgmessageobject.h>
#include "defs.h"
#include "fight.h"
#include "action.h"
#include "city.h"
#include "armyset.h"
#include "xmlhelper.h"
#include <sigc++/sigc++.h>

class Stacklist;
class Hero;

/** The abstract player class.
  *
  * This class does not yet implement an actual player type. Its purpose is to 
  * provide an abstract class design which every player implementation has to
  * fulfill. Of each player class we demand the following functionality:
  *
  * 1. functions for _every_ action a player can do
  * 2. some kind of callback functions if the player has a choice (e.g. if he has
  *    conquered a city, he may choose between razing, pillaging and occupying)
  * 3. signals which are raised whenever something important happens
  * 4. an actionlist which keeps track of _everything_ a player has done
  * 
  * The fourth point allows us an easy network playing scheme. After a player
  * has finished, all he has to do is sending his actionlist to the over network
  * players. Since every item which the player can touch/kill etc. (cities,
  * armies, ruins,...) has a unique id, the remote game instances can then
  * simply apply these action lists to their own situation.
  *
  * The third point allows us to dock other classes to every possible event. One
  * example are ruin searching quests which must be informed whenever any
  * player searches a ruin. Or the bigmap can be informed when a stack moves,
  * so that it may update its contents.
  *
  * The second item allows an easy coexistence of AI and human players.
  * Basically, an AI player just follows some kind of routine while a human
  * player uses gui interaction. However, this becomes a bit problematic
  * whenever the player may decide something. One solution is providing
  * callbacks for these cases. The human player then opens a dialog somehow
  * while the AI player overwrites the default behaviour.
  *
  * The first point makes a nice derivation scheme possible. It is possible to
  * divide the player types into a local player, who implements all these
  * functions and a networked player. The latter one simply overwrites these
  * functions so that the game status is updated for each item of the actionlist
  * he has been sent. Furthermore, with a local player having been implemented,
  * it is extremely easy to write an AI player. All you have to do is overwrite
  * the startTurn function with your own code. For every action you already know
  * there is an implementation in a superior class which takes off the burden
  * for the actual work.
  *
  * 
  * Finally, the current derivation scheme is as follows
  * 
  * Player
  *   |
  *   +--- NetworkPlayer (not implemented yet)
  *   |
  *   +--- RealPlayer (a local player, human players use instances of this class)
  *          +
  *          |
  *          +--- AI_Dummy (AI for the neutral player, does nothing)
  *          |
  *          +--- AI_Fast (dumb AI, tries to conquer the nearest city)
  */

class Player : public SigC::Object
{
    public:
        //! The available player types. Needed when loading a player.
        enum TYPE {HUMAN = 0, AI_FAST = 1, AI_DUMMY = 2, NEUTRAL = 3};
        
        /** Default constructor
          * 
          * @param name         the name of the player
          * @param armyset      the player's armyset
          * @param color        the player's main color
          */
        Player (std::string name, std::string armyset, SDL_Color color,
            TYPE type);

        //! Copy constructor
        Player(const Player&);

        //! Constructor for loading. See XML_Helper for documentation.
        Player(XML_Helper* helper);
        virtual ~Player();
        
        //! Change the player's armyset
        void setArmyset(std::string armyset){d_armyset = armyset;}

        //! Set the type of the player (to be used by derived classes only)
        void setType(TYPE type) {d_type = type;}

        //! Add gold to the player's treasure
        void addGold(int gold);

        //! Subtract gold from the player's treasure
        void withdrawGold(int gold);

        //! Remove all items from the list of player's actions
        void clearActionlist();

        //! Add a stack to the player's list of stacks
        void addStack(Stack* stack);

        //! Remove a stack from the player's stacklist
        bool deleteStack(Stack* stack);
        
        
        //! Returns the unique ID of the player
        unsigned int getId()    {return d_id;}

        //! Returns the list of player's actions. Attention: Shallow copy!!
        std::list<Action*> getActionlist()      {return d_actions;}

        //! Return the player's armyset
        Armyset* getArmyset();

        //! Return whether the player has been killed
        bool isDead() {return d_dead;}
        
        //! Return the type of the player
        TYPE getType()          {return d_type;}

        //! Returns the main color of the player
        SDL_Color getColor() const              {return d_color;}

        //! Returns the color palette derived from the player's main color
        SDL_Color* getPalette()                 {return d_palette;}

        //! Returns the gold of the player
        int getGold()           {return d_gold;}

        //! Returns the name of the player
        std::string getName()   {return d_name;}

        //! Returns the list of stacks owned by the player
        Stacklist* getStacklist()               {return d_stacklist;}
        
        //! A shortcut for getting the stack marked as active
        Stack* getActivestack();
        
        
        //! Derive the player palette from the player's main color
        void initPalette();

        //! Dumps the items in the actionlist on stderr
        void dumpActionlist();

        //! Checks if a stack has enough moves for its next step
        bool enoughMoves(Stack* s);
       
        //! Mark the player as dead; this kills all his armies etc.
        void kill();
        
        
        //! Saves the player data
        bool save(XML_Helper* helper);

        /** Loads a player
          * 
          * This is a bit inconsistent with other classes, but with players you
          * have the problem that there are different types with different
          * classes. So we need a static member function which looks which
          * player type to load and calls the constructor of the appropriate
          * class.
          *
          * @param helper       the XML_Helper instance of the savegame
          */
        static Player* loadPlayer(XML_Helper* helper);
        
        //! Loads the subdata of a player (actions and stacklist)
        bool load(std::string tag, XML_Helper* helper);
        
        
        /** This function is called when a player's turn starts. For AI players,
          * this function should start the algorithm.
          *
          * @return true if wverything went well
          */
        virtual bool startTurn() = 0;

        /** Called so that the player can decide what to do with an
            occupied city as soon as he has occupied it (mainly for AI)

            @return always returns true (unused feature)
            @param  city   the conquered city
          */
        virtual bool invadeCity(City* c) = 0;

        /** Called whenever a hero decides to join you, so the player can
            decide what to do with the hero.

            @return true if the player accepts the hero, false otherwise
            @param  hero    the hero who has offered his service
            @param  cost    the amount of gold neccessary to recruit the
                            hero

            @note Don't touch the hero except for naming purposes etc.
                  The hero is integrated elsewhere!
          */
        virtual bool recruitHero(Hero* hero, int cost) = 0;
        
        /** Called to split the stack s in two stacks. You have to take great
          * care that one of the created stacks moves away after that. Some code
          * parts assume that each tile is occupied by just one stack!
          * You can change which armies are in the new stack by setting their
          * "grouped" value.
          *
          * @param s        the stack to split
          * @return false if there were any problems, else true
          */
        virtual bool stackSplit(Stack* s) =0;
        
        /** Called to merge two stacks into one.
          * 
          * @param receiver         the receiving stack
          * @param joining          the joining stack, destroyed after the join
          * @return false if an error occured, else true
          */
        virtual bool stackJoin(Stack* receiver, Stack* joining) =0;

        //! A shortcut for stackMove(s, <last path item>, true)
        virtual bool stackMove(Stack* s) =0;

        /** Move a stack to a specified position
          * 
          * Moves a stack. The path is calculated on the fly unless you set
          * follow to true. In this case, an existing path is checked and
          * iterated over. This is useful if a stack didn't reach its target
          * within one round and should continue the movement.
          *
          * @param s                the stack to be moved
          * @param dest             the destination of the move
          * @param follow           if set to false, calculate the path
          * @return false on error, true otherwise
          */
        virtual bool stackMove(Stack* s, PG_Point dest, bool follow)=0;
        
        /** Two stacks fight each other.
          * 
          * @param sattacker        the attacking stack
          * @param sdefender        the defending stack
          * @return one of Fight::ATTACKER_WON, Fight::DEFENDER_WON, Fight::NONE
          */
        virtual Fight::RESULT stackFight(Stack* sattacker, Stack* sdefender) =0;

        /** Some stacks fight a city
          * 
          * During an attack on a city, more than one stack may participate.
          * That is why the attacker is a list of stacks. The defenders have to
          * be collected from the armies stationed within the city.
          *
          * @param attackers        list of attacking stack
          * @param target           the attacked city
          * @return one of Fight::ATTACKER_WON, Fight::DEFENDER_WON, Fight::NONE
          */
        virtual Fight::RESULT stackFightCity(std::vector<Stack*> attackers, City* target) =0;

        /** A stack searches a ruin. The stack should contain a hero.
          *
          * @param s                the stack which searches the ruin
          * @param r                the ruin to be searched
          * @return true if the ruin was searched (keeper defeated)
          */
        virtual bool stackSearchRuin(Stack* s, Ruin* r) =0;

        /** A stack visits a temple and becomes blessed. By blessing, the attack
          * strength of all armies rises by 1.
          *
          * @param s                the stack visiting the temple
          * @param t                the visited temple
          * @return false on error, true otherwise
          */
        virtual bool stackVisitTemple(Stack* s, Temple* t) =0;

        /** Occupy a city (i.e. change the owner to yourself)
          * 
          * @param c                the occupied city
          * @return false on error, true otherwise
          */
        virtual bool cityOccupy(City* c) =0;

        /** Pillage a city (destroy the latest production and get some gold)
          * 
          * @param c                the city to be pillaged
          * @return false on error, true otherwise
          */
        virtual bool cityPillage(City* c) = 0;

        /** Raze (permanently destroy) a city
          * 
          * @param c                the city to be razed
          * @return false on error, true otherwise
          */
        virtual bool cityRaze(City* c) =0;

        /** Upgrade the defense level of a city
          * 
          * @param c                the city to be upgraded
          * @return false on error, tru otherwise
          */
        virtual bool cityUpgradeDefense(City* c) =0;

        /** Add another production to a city
          * 
          * @param c                the lucky city
          * @param armytype         the index of the army type to add
          */
        virtual bool cityBuyProduction(City* c, int armytype) =0;

        /** Change the production of a city
          * 
          * @param c                the affected city
          * @param production       the index of the selected production slot
          * @return false on error, true otherwise
          */
        virtual bool cityChangeProduction(City* c, int production) =0;

        /** Gives the player some random reward
          * 
          * @param level            the quality of the reward
          * @param s                the stack which has caused the reward
          * @param reward           a pointer for storing the type of the reward
          * @return false on error, true otherwise
          */
        virtual bool giveReward (int level, Stack* s, fl_Treasure* reward = 0) = 0;
        
        /** Signal raised when a city is conquered. This signal is solely
          * for internal use. Don't use it, you may break stability (Background:
          * libsigc++ doesn't guarantee order of execution and this signal may
          * usually end the game which can have unwanted side effects).
          */
        SigC::Signal1<void, City*> sinvadingCity;

        //! Signal raised whenever a player has conquered a city. This is the
        //! signal you should use for further actions.
        SigC::Signal1<void, City*> soccupyingCity;
        
        //! Signal raised when a hero is recruited
        SigC::Signal2<bool, Hero*, int> srecruitingHero;

        //! Signal raised whenever the player successfully searched a ruin
        SigC::Signal2<void, Player*, Ruin*> ssearchingRuin;
        
        //! Signal raised whenever a player's status (e.g. gold) changes
        SigC::Signal0<void> schangingStatus;
        //! Signal raised whenever the stack's staus has changed
        SigC::Signal1<void, Stack*> supdatingStack;
        //! Signal raised whenever the status of a city has changed
        SigC::Signal1<void, City*> supdatingCity;
        //! Signal raised whenever the player wants to interrupt some timers
        SigC::Signal0<void> sinterruptTimers;
        //! Signal raised whenever the player asks to restart timers
        SigC::Signal0<void> scontinueTimers;
        
    protected:
        //! Move stack s one step forward on his stored path
        virtual bool stackMoveOneStep(Stack* s) = 0;

        // DATA
        SDL_Color d_color;
        SDL_Color* d_palette;
        std::string d_name;
        std::string d_armyset;    // name of the armyset
        int d_gold;
        bool d_dead;            // is player already dead?
        TYPE d_type;
        unsigned int d_id;
        std::list<Action*> d_actions; //list of actions done by the player
        Stacklist* d_stacklist;
};

extern SigC::Signal1<void, Player::TYPE>  sendingTurn;

#endif // PLAYER_H

// End of file
