//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PLAYERLIST_H
#define PLAYERLIST_H

#include <list>
#include <string>
#include <pgmessageobject.h>
#include "player.h"
#include <sigc++/object_slot.h>

/** List of all players of a scenario
  * 
  * The playerlist is also implemented as a singleton function. The currently
  * active player is designated, you can access players by name or id and the
  * playerlist can check if there are more than one player remaining alive.
  */

class Playerlist : public std::list<Player*>, public SigC::Object
{
    public:

        //! Gets the singleton instance or creates a new one
        static Playerlist* getInstance();

        //! Loads the playerlist (including all players)
        static Playerlist* getInstance(XML_Helper* helper);

        //! Explicitely deletes the singleton instance
        static void deleteInstance();

        //! Returns the active player of the singleton instance
        static Player* getActiveplayer() {return d_activeplayer;}
        
        
        //! Sets the active player to the next player in the order
        void nextPlayer();

        /** Checks if a player has died (==doesn't own a city). If so, it
          * removes the player from the list and displys a message box.
          */
        void checkPlayers();

        /** sets the neutral player. Though the neutral player is located in the
          * list of existing players, it is handled in a special way. His colors
          * are e.g. used for stacks not owned by a player (such as monsters
          * infesting ruins), which is why we keep track of the neutral player
          * with a special pointer.
          *
          * @param neutral          the new neutral player
          */
        void setNeutral(Player* neutral) {d_neutral = neutral;}

        //! Returns the neutral player
        Player* getNeutral() const {return d_neutral;}

        //! Returns player named "name" or none if not existent
        Player* getPlayer(std::string name);

        //! Returns player with the specified id or none if not existent
        Player* getPlayer(unsigned int id);

        //! Returns the id of player p
        int getPlayerNr(Player* p);

        //! Returns the number of living players (neutral player excluded)
        int getNoOfPlayers();

        /** Returns the first player alive (needed to determine the begin of
          * a new round).
          */
        Player* getFirstLiving();
        
        
        //! Saves the playerlist. See XML_Helper for further docu.
        bool save(XML_Helper* helper);

        //! Callback for loading the playerlist.
        bool load(std::string, XML_Helper* helper);

        //! This signal is emitted when a player has died.
        SigC::Signal1<void, Player*> splayerDead;
    
    protected:
        // CREATORS
        Playerlist();
        Playerlist(XML_Helper* helper);
        ~Playerlist();
        
    private:
        /** Erase an item. This needs to be implemented, because we have to free
          * the pointer. Else it behaves like the STL erase member functions.
          */
        iterator fl_erase(iterator it);

        //static pointers for the singleton
        static Playerlist* s_instance;
        
        // DATA
        static Player* d_activeplayer;
        Player* d_neutral;
};

#endif // PLAYERLIST_H
